__author__ = 'Daniel Harmsworth'

from tkinter import *
from serial import *
import glob, sys, threading


serial_data = []
captured_packets = []

class SerialMonitor(threading.Thread):

    port = ""
    signal = False
    kill = False

    def run(self):
        while not self.kill:
            if self.signal:
                print("Connecting to Serial...")
                serial = Serial(self.port, 9600)
                serial.timeout = 0   # Needs to be in non-blocking mode
                while self.signal and not self.kill:
                    if serial.inWaiting() == 10:
                        welly_message = serial.read(10)
                        serial_data.append(welly_message)

def build_packet_info(self):
    pi_frame = Frame(self)
    pi_frame.grid(column = 1, row=5, columnspan = 6)
    pi_frame.config()

def main():
    def packet_info(evt):
        index = int(packets.curselection()[0])

        if captured_packets[index][2] == 0x54:
            message_type.config(text="Diagnostic")
        else:
            print(index)

    def set_port(port_name):
        if SerialMonitor.signal:
            toggle_capture()

        print("DEBUG: Changing port to ", port_name)
        SerialMonitor.port = port_name

    def toggle_capture():
        if SerialMonitor.signal:
            SerialMonitor.signal = False
            start_stop["text"] = "Start Capture"
        else:
            start_stop["text"] = "Stop Capture"
            SerialMonitor.signal = True

    def service_loop():
        if serial_data:
            for i in serial_data:
                item = serial_data.pop(0)
                formatted = []

                for b in item:
                    formatted.append(b)

                packets.insert(END, formatted)

                check = wellynet_checksum(item)
                if check == 0:
                    packets.itemconfig(packets.size()-1, bg='green')
                else:
                    packets.itemconfig(packets.size()-1, bg='red')

                captured_packets.append(item)

        root.after(10, service_loop)

    def wellynet_checksum(packet):
        lrc = 0
        if not len(packet) == 10:
            return 1
        else:
            for b in range(0,9):
                lrc = (lrc + int(packet[b])) & 0xFF
            lrc = ((lrc ^ 0xFF)+1) & 0xFF
        if lrc == packet[9]:
            return 0
        else:
            return lrc

    # Main Code Block

    SerialMonitor().start()
    root = Tk()
    root.geometry("600x600+300+300")
    root.wm_title("WellyNet Test Tool")

    menu_bar = Menu(root)
    root.config(menu=menu_bar)

    root.columnconfigure(1, weight=1)
    root.columnconfigure(2, weight=1)
    root.columnconfigure(3, weight=1)
    root.columnconfigure(4, weight=1)
    root.columnconfigure(5, weight=1)

    root.rowconfigure(1, weight=1)
    root.rowconfigure(2, weight=1)
    root.rowconfigure(3, weight=1)
    root.rowconfigure(4, weight=1)
    root.rowconfigure(5, weight=1)

    # Create menu bar
    file_menu = Menu(menu_bar)
    file_menu.add_command(label="Exit", command=root.quit)
    menu_bar.add_cascade(label="File", menu=file_menu)

    port_menu = Menu(menu_bar)
    port_list = serial_ports()

    for i in port_list:
        port_menu.add_command(label=i, command=lambda i=i: set_port(i))

    menu_bar.add_cascade(label="Ports", menu=port_menu)

    start_stop = Button(root, text="Start Capture", command=toggle_capture)
    start_stop.grid(row=1, column=1)

    packets = Listbox(root)
    packets.grid(row=1, column=2, columnspan=5, rowspan=2, sticky=E+W+S+N, pady=10, padx=10)
    packets.bind('<<ListboxSelect>>', packet_info)

    build_packet_info(root)

    root.after(10, service_loop)
    root.mainloop()
    print("Killing Serial Monitor Thread")
    SerialMonitor.kill = True


def serial_ports():
    """Lists serial ports

    :raises EnvironmentError:
        On unsupported or unknown platforms
    :returns:
        A list of available serial ports
    """
    if sys.platform.startswith('win'):
        ports = ['COM' + str(i + 1) for i in range(256)]

    elif sys.platform.startswith('linux') or sys.platform.startswith('cygwin'):
        # this is to exclude your current terminal "/dev/tty"
        ports = glob.glob('/dev/tty[A-Za-z]*')

    elif sys.platform.startswith('darwin'):
        ports = glob.glob('/dev/tty.*')

    else:
        raise EnvironmentError('Unsupported platform')

    result = []
    for available_ports in ports:
        try:
            s = Serial(available_ports)
            s.close()
            result.append(available_ports)
        except (OSError, SerialException):
            pass
    return result


#def calc_checksum(wn_packet):


if __name__ == '__main__':
    main()